@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.c,v 1.68 2005/12/12 17:27:56 pborelli Exp $
+ * $Id: gedit-utils.c,v 1.69 2005/12/29 15:13:39 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -294,7 +294,7 @@ gedit_utils_uri_exists (const gchar* text_uri)
 	return res;
 }
 
-gchar* 
+gchar *
 gedit_utils_escape_search_text (const gchar* text)
 {
 	GString *str;
@@ -495,6 +495,85 @@ gedit_warning (GtkWindow *parent, const gchar *format, ...)
 	g_free (str);
 }
 
+/*
+ * Doubles underscore to avoid spurious menu accels.
+ */
+gchar * 
+gedit_utils_escape_underscores (const gchar* text,
+				gssize       length)
+{
+	GString *str;
+	const gchar *p;
+	const gchar *end;
+
+	g_return_val_if_fail (text != NULL, NULL);
+
+	if (length < 0)
+		length = strlen (text);
+
+	str = g_string_sized_new (length);
+
+	p = text;
+	end = text + length;
+
+	while (p != end)
+	{
+		const gchar *next;
+		next = g_utf8_next_char (p);
+
+		switch (*p)
+		{
+			case '_':
+				g_string_append (str, "__");
+				break;
+			default:
+				g_string_append_len (str, p, next - p);
+				break;
+		}
+
+		p = next;
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+/*
+ * Replaces '/' with '-' to avoid problems in xml paths.
+ */
+gchar *
+gedit_utils_escape_slashes (const gchar *text,
+			    gssize       length)
+{
+	GString *str;
+	const gchar *p;
+	const gchar *end;
+
+	g_return_val_if_fail (text != NULL, NULL);
+
+	if (length < 0)
+		length = strlen (text);
+
+	str = g_string_sized_new (length);
+
+	p = text;
+	end = text + length;
+
+	while (p != end)
+	{
+		const gchar *next;
+		next = g_utf8_next_char (p);
+
+		if (*p == '/')
+			g_string_append (str, "-");
+		else
+			g_string_append_len (str, p, next - p);
+
+		p = next;
+	}
+
+	return g_string_free (str, FALSE);
+}
+
 /* the following functions are taken from eel */
 
 gchar *
