@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c,v 1.5 2005/12/12 17:27:56 pborelli Exp $
+ * $Id: gedit-window.c,v 1.9 2006/01/02 14:26:40 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,7 +62,9 @@
 #include "recent-files/egg-recent-view-gtk.h"
 #include "recent-files/egg-recent-view-uimanager.h"
 
-#define GEDIT_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_WINDOW, GeditWindowPrivate))
+#define GEDIT_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object),\
+					 GEDIT_TYPE_WINDOW,                    \
+					 GeditWindowPrivate))
 
 /* Signals */
 enum
@@ -385,19 +387,19 @@ set_toolbar_style (GeditWindow *window,
 		gtk_widget_show (window->priv->toolbar);
 	else
 		gtk_widget_hide (window->priv->toolbar);
-	
-	action = gtk_action_group_get_action (window->priv->action_group,
+
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewToolbar");
-					      
+
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-	
+
 	/* Set style */
 	if (origin == NULL)
 		style = gedit_prefs_manager_get_toolbar_buttons_style ();
 	else
 		style = origin->priv->toolbar_style;
-		
+	
 	window->priv->toolbar_style = style;
 		
 	switch (style)
@@ -407,21 +409,21 @@ set_toolbar_style (GeditWindow *window,
 			gtk_toolbar_unset_style (
 					GTK_TOOLBAR (window->priv->toolbar));
 			break;
-			
+
 		case GEDIT_TOOLBAR_ICONS:
 			gedit_debug_message (DEBUG_WINDOW, "GEDIT: ICONS");
 			gtk_toolbar_set_style (
 					GTK_TOOLBAR (window->priv->toolbar),
 					GTK_TOOLBAR_ICONS);
 			break;
-			
+
 		case GEDIT_TOOLBAR_ICONS_AND_TEXT:
 			gedit_debug_message (DEBUG_WINDOW, "GEDIT: ICONS_AND_TEXT");
 			gtk_toolbar_set_style (
 					GTK_TOOLBAR (window->priv->toolbar),
 					GTK_TOOLBAR_BOTH);			
 			break;
-			
+
 		case GEDIT_TOOLBAR_ICONS_BOTH_HORIZ:
 			gedit_debug_message (DEBUG_WINDOW, "GEDIT: ICONS_BOTH_HORIZ");
 			gtk_toolbar_set_style (
@@ -453,7 +455,7 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 
 	view = gedit_tab_get_view (tab);
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-	
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "FileSave");
 	gtk_action_set_sensitive (action,
@@ -594,6 +596,24 @@ language_toggled (GtkToggleAction *action,
 	gedit_document_set_language (doc, (GtkSourceLanguage *) lang);
 }
 
+static gchar *
+escape_section_name (gchar *name)
+{
+	gchar *tmp;
+	gchar *ret;
+	gssize len;
+
+	len = strlen (name);
+
+	/* escape slashes doesn't change the string lenght */
+	tmp = gedit_utils_escape_slashes (name, len);
+	ret = g_markup_escape_text (tmp, len);
+
+	g_free (tmp);
+
+	return ret;
+}
+
 static void
 create_language_menu_item (GtkSourceLanguage *lang,
 			   gint               index,
@@ -605,20 +625,22 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	GtkAction *normal_action;
 	GSList *group;
 	gchar *section;
+	gchar *escaped_section;
 	gchar *lang_name;
+	gchar *escaped_lang_name;
 	gchar *tip;
 	gchar *path;
 
 	section = gtk_source_language_get_section (lang);
+	escaped_section = escape_section_name (section);
 
 	/* check if the section submenu exists or create it */
 	section_action = gtk_action_group_get_action (window->priv->languages_action_group,
-						      section);
+						      escaped_section);
 
 	if (section_action == NULL)
 	{
-		// CHECK: escaping strings
-		section_action = gtk_action_new (section,
+		section_action = gtk_action_new (escaped_section,
 						 section,
 						 NULL,
 						 NULL);
@@ -630,19 +652,21 @@ create_language_menu_item (GtkSourceLanguage *lang,
 		gtk_ui_manager_add_ui (window->priv->manager,
 				       ui_id,
 				       "/MenuBar/ViewMenu/ViewHighlightModeMenu/LanguagesMenuPlaceholder",
-				       section, section,
+				       escaped_section, escaped_section,
 				       GTK_UI_MANAGER_MENU,
 				       FALSE);
 	}
 
 	/* now add the language item to the section */
 	lang_name = gtk_source_language_get_name (lang);
+
+	escaped_lang_name = g_markup_escape_text (lang_name, -1);
+
 	tip = g_strdup_printf (_("Use %s highlight mode"), lang_name);
 	path = g_strdup_printf ("/MenuBar/ViewMenu/ViewHighlightModeMenu/LanguagesMenuPlaceholder/%s",
-				section);
+				escaped_section);
 
-	// CHECK: escaping strings
-	action = gtk_radio_action_new (lang_name,
+	action = gtk_radio_action_new (escaped_lang_name,
 				       lang_name,
 				       tip,
 				       NULL,
@@ -666,14 +690,16 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	gtk_ui_manager_add_ui (window->priv->manager,
 			       ui_id,
 			       path,
-			       lang_name, lang_name,
+			       escaped_lang_name, escaped_lang_name,
 			       GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
 
 	g_free (path);
 	g_free (tip);
 	g_free (lang_name);
+	g_free (escaped_lang_name);
 	g_free (section);
+	g_free (escaped_section);
 }
 
 static void
@@ -734,6 +760,7 @@ update_languages_menu (GeditWindow *window)
 	GtkAction *action;
 	GtkSourceLanguage *lang;
 	gchar *lang_name;
+	gchar *escaped_lang_name;
 
 	doc = gedit_window_get_active_document (window);
 	if (doc == NULL)
@@ -745,6 +772,8 @@ update_languages_menu (GeditWindow *window)
 	else
 		lang_name = g_strdup ("LangNormal");
 
+	escaped_lang_name = g_markup_escape_text (lang_name, -1);
+
 	actions = gtk_action_group_list_actions (window->priv->languages_action_group);
 
 	/* prevent recursion */
@@ -756,7 +785,7 @@ update_languages_menu (GeditWindow *window)
 	}
 
 	action = gtk_action_group_get_action (window->priv->languages_action_group,
-					      lang_name);
+					      escaped_lang_name);
 
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
@@ -769,6 +798,7 @@ update_languages_menu (GeditWindow *window)
 
 	g_list_free (actions);
 	g_free (lang_name);
+	g_free (escaped_lang_name);
 }
 
 static void
@@ -915,6 +945,10 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 				      gedit_always_sensitive_menu_entries,
 				      G_N_ELEMENTS (gedit_always_sensitive_menu_entries),
 				      window);
+	gtk_action_group_add_toggle_actions (action_group,
+					     gedit_always_sensitive_toggle_menu_entries,
+					     G_N_ELEMENTS (gedit_always_sensitive_toggle_menu_entries),
+					     window);
 
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	g_object_unref (action_group);
@@ -926,10 +960,6 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 				      gedit_menu_entries,
 				      G_N_ELEMENTS (gedit_menu_entries),
 				      window);
-	gtk_action_group_add_toggle_actions (action_group,
-					     gedit_toggle_menu_entries,
-					     G_N_ELEMENTS (gedit_toggle_menu_entries),
-					     window);
 
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	g_object_unref (action_group);
@@ -949,6 +979,17 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 	action = gtk_action_group_get_action (action_group, "EditUndo");
 	g_object_set (action, "is_important", TRUE, NULL);
 
+	action_group = gtk_action_group_new ("GeditQuitWindowActions");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group,
+				      gedit_quit_menu_entries,
+				      G_N_ELEMENTS (gedit_quit_menu_entries),
+				      window);
+
+	gtk_ui_manager_insert_action_group (manager, action_group, 0);
+	g_object_unref (action_group);
+	window->priv->quit_action_group = action_group;
+
 	/* now load the UI definition */
 	if (gtk_ui_manager_add_ui_from_file (manager, "gedit-ui.xml", NULL)) // REMOVE ME... just to allow running without install
 		;
@@ -1105,8 +1146,9 @@ update_documents_list_menu (GeditWindow *window)
 		GtkWidget *tab;
 		GtkRadioAction *action;
 		gchar *action_name;
-		gchar *tab_name; //CHECK: must escape underscores and gmarkup
-		gchar *tip;	 // ditto as above
+		gchar *tab_name;
+		gchar *name;
+		gchar *tip;
 		gchar *accel;
 
 		tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (p->notebook), i);
@@ -1120,13 +1162,14 @@ update_documents_list_menu (GeditWindow *window)
 		 */
 		action_name = g_strdup_printf ("Tab_%d", i);
 		tab_name = _gedit_tab_get_name (GEDIT_TAB (tab));
+		name = gedit_utils_escape_underscores (tab_name, -1);
 		tip =  g_strdup_printf (_("Activate %s"), tab_name);
 
 		/* alt + 1, 2, 3... 0 to switch to the first ten tabs */
 		accel = (i < 10) ? g_strdup_printf ("<alt>%d", (i + 1) % 10) : NULL;
 
 		action = gtk_radio_action_new (action_name,
-					       tab_name,
+					       name,
 					       tip,
 					       NULL,
 					       i);
@@ -1160,6 +1203,7 @@ update_documents_list_menu (GeditWindow *window)
 
 		g_free (action_name);
 		g_free (tab_name);
+		g_free (name);
 		g_free (tip);
 		g_free (accel);
 	}
@@ -1186,7 +1230,7 @@ set_statusbar_style (GeditWindow *window,
 	else
 		gtk_widget_hide (window->priv->statusbar);
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewStatusbar");
 					      
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -1507,9 +1551,7 @@ set_sensitivity_according_to_window_state (GeditWindow *window)
 	   the same time (may be we can remove this limitation in the future) */
 	/* We disable File->Quit/CloseAll if state is saving since saving cannot be
 	   cancelled (may be we can remove this limitation in the future) */
-	action = gtk_action_group_get_action (window->priv->action_group,
-				              "FileQuit");
-	gtk_action_set_sensitive (action, 
+	gtk_action_group_set_sensitive (window->priv->quit_action_group,
 				  !(window->priv->state & GEDIT_WINDOW_STATE_SAVING) &&
 				  !(window->priv->state & GEDIT_WINDOW_STATE_PRINTING));
 
@@ -1518,7 +1560,7 @@ set_sensitivity_according_to_window_state (GeditWindow *window)
 	gtk_action_set_sensitive (action, 
 				  !(window->priv->state & GEDIT_WINDOW_STATE_SAVING) &&
 				  !(window->priv->state & GEDIT_WINDOW_STATE_PRINTING));
-				  
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 				              "FileSaveAll");
 	gtk_action_set_sensitive (action, 
@@ -1554,17 +1596,24 @@ set_sensitivity_according_to_window_state (GeditWindow *window)
 
 	if ((window->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) != 0)
 	{
-		// FIXME: View menu and Find should be active when in SAVING_SESSION state
-		
+		/* TODO: If we really care, Find could be active
+		 * when in SAVING_SESSION state */
+
 		if (gtk_action_group_get_sensitive (window->priv->action_group))
 			gtk_action_group_set_sensitive (window->priv->action_group,
 							FALSE);
+		if (gtk_action_group_get_sensitive (window->priv->quit_action_group))
+			gtk_action_group_set_sensitive (window->priv->quit_action_group,
+							FALSE);
 	}
 	else
 	{
 		if (!gtk_action_group_get_sensitive (window->priv->action_group))
 			gtk_action_group_set_sensitive (window->priv->action_group,
 							window->priv->num_tabs > 0);
+		if (!gtk_action_group_get_sensitive (window->priv->quit_action_group))
+			gtk_action_group_set_sensitive (window->priv->quit_action_group,
+							window->priv->num_tabs > 0);
 	}
 }
 
@@ -1655,7 +1704,7 @@ sync_state (GeditTab *tab, GParamSpec *pspec, GeditWindow *window)
 		return;
 
 	set_sensitivity_according_to_tab (window, tab);
-	
+
 	g_signal_emit (G_OBJECT (window), signals[ACTIVE_TAB_STATE_CHANGED], 0);		
 }
 
@@ -1664,7 +1713,8 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GeditWindow *window)
 {
 	GtkAction *action;
 	gchar *action_name;
-	gchar *tab_name; // CHECK escaping
+	gchar *tab_name;
+	gchar *escaped_name;
 	gchar *tip;
 	gint n;
 	GeditDocument *doc;
@@ -1683,13 +1733,15 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GeditWindow *window)
 	g_return_if_fail (action != NULL);
 
 	tab_name = _gedit_tab_get_name (tab);
+	escaped_name = gedit_utils_escape_underscores (tab_name, -1);
 	tip =  g_strdup_printf (_("Activate %s"), tab_name);
 
-	g_object_set (action, "label", tab_name, NULL);
+	g_object_set (action, "label", escaped_name, NULL);
 	g_object_set (action, "tooltip", tip, NULL);
 
 	g_free (action_name);
 	g_free (tab_name);
+	g_free (escaped_name);
 	g_free (tip);
 
 	doc = gedit_tab_get_document (tab);
@@ -2020,10 +2072,9 @@ notebook_tab_added (GeditNotebook *notebook,
 	gtk_action_set_sensitive (action,
 				  window->priv->num_tabs > 1);
 
-
 	view = gedit_tab_get_view (tab);
 	doc = gedit_tab_get_document (tab);
-	
+
 	/* IMPORTANT: remember to disconnect the signal in notebook_tab_removed
 	 * if a new signal is connected here */
 
@@ -2204,13 +2255,10 @@ notebook_tab_removed (GeditNotebook *notebook,
 			gtk_action_group_set_sensitive (window->priv->action_group,
 							FALSE);
 
-		// FIXME: Quit and the view menu should be active 
-		
 		action = gtk_action_group_get_action (window->priv->action_group,
 						      "ViewHighlightMode");
-
 		gtk_action_set_sensitive (action, FALSE);
-		
+
 		gedit_plugins_engine_update_plugins_ui (window, FALSE);
 	}
 
@@ -2260,7 +2308,7 @@ notebook_tab_close_request (GeditNotebook *notebook,
 			    GeditTab      *tab,
 			    GtkWindow     *window)
 {
-	/* CHECK: we are destroying the tab before the default handler
+	/* Note: we are destroying the tab before the default handler
 	 * seems to be ok, but we need to keep an eye on this. */
 	_gedit_cmd_file_close_tab (tab, GEDIT_WINDOW (window));
 }
@@ -2360,7 +2408,7 @@ create_side_panel (GeditWindow *window)
 
 	gedit_debug (DEBUG_WINDOW);
 
-	window->priv->side_panel = gedit_panel_new ();
+	window->priv->side_panel = gedit_panel_new (GTK_ORIENTATION_VERTICAL);
 
 	gtk_paned_pack1 (GTK_PANED (window->priv->hpaned), 
 			 window->priv->side_panel, 
@@ -2389,7 +2437,7 @@ create_side_panel (GeditWindow *window)
 
 	visible = gedit_prefs_manager_get_side_pane_visible ();
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewSidePane");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -2422,7 +2470,8 @@ create_bottom_panel (GeditWindow *window)
 
 	gedit_debug (DEBUG_WINDOW);
 
-	window->priv->bottom_panel = gedit_panel_new ();
+	window->priv->bottom_panel = gedit_panel_new (GTK_ORIENTATION_HORIZONTAL);
+	
 	gtk_paned_pack2 (GTK_PANED (window->priv->vpaned), 
 			 window->priv->bottom_panel, 
 			 FALSE, 
@@ -2443,7 +2492,7 @@ create_bottom_panel (GeditWindow *window)
 
 	visible = gedit_prefs_manager_get_bottom_panel_visible();
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewBottomPanel");
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -2734,7 +2783,6 @@ gedit_window_close_tab (GeditWindow *window,
 				   tab);
 }
 
-// CHECK: these should be priv? pbor.
 void
 gedit_window_close_all_tabs (GeditWindow *window)
 {
@@ -2802,7 +2850,7 @@ _gedit_window_set_statusbar_visible (GeditWindow *window,
 	if (gedit_prefs_manager_statusbar_visible_can_set ())
 		gedit_prefs_manager_set_statusbar_visible (visible);
 		
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewStatusbar");		
 		
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -2837,7 +2885,7 @@ _gedit_window_set_toolbar_visible (GeditWindow *window,
 	if (gedit_prefs_manager_toolbar_visible_can_set ())
 		gedit_prefs_manager_set_toolbar_visible (visible);
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewToolbar");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -2882,7 +2930,7 @@ _gedit_window_set_side_panel_visible (GeditWindow *window,
 	if (gedit_prefs_manager_side_pane_visible_can_set ())
 		gedit_prefs_manager_set_side_pane_visible (visible);
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewSidePane");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -2933,7 +2981,7 @@ _gedit_window_set_bottom_panel_visible (GeditWindow *window,
 	if (gedit_prefs_manager_bottom_panel_visible_can_set ())
 		gedit_prefs_manager_set_bottom_panel_visible (visible);
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewBottomPanel");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
